@@ -2,7 +2,7 @@
  *
  * rdf_storage.c - RDF Storage Implementation
  *
- * $Id: rdf_storage.c,v 1.46 2003/08/27 10:36:48 cmdjb Exp $
+ * $Id: rdf_storage.c,v 1.48 2003/09/04 10:06:56 cmdjb Exp $
  *
  * Copyright (C) 2000-2003 David Beckett - http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology - http://www.ilrt.org/
@@ -245,6 +245,41 @@ librdf_new_storage (librdf_world *world,
 }
 
 
+/**
+ * librdf_new_storage_with_options - Constructor - create a new librdf_storage object
+ * @world: redland world object
+ * @storage_name: the storage factory name
+ * @name: an identifier for the storage
+ * @options: &librdf_hash of options to use
+ *
+ * The options can be NULL if none are required.
+ *
+ * Return value: a new &librdf_storage object or NULL on failure
+ */
+librdf_storage*
+librdf_new_storage_with_options (librdf_world *world, 
+                                 char *storage_name, char *name, 
+                                 librdf_hash *options) {
+  librdf_storage_factory* factory;
+  librdf_hash* options_hash;
+  
+  factory=librdf_get_storage_factory(storage_name);
+  if(!factory)
+    return NULL;
+
+  options_hash=librdf_new_hash_from_hash(options);
+  if(!options_hash)
+    return NULL;
+
+  if(librdf_hash_open(options_hash, NULL, 0, 1, 1, NULL)) {
+    librdf_free_hash(options_hash);
+    return NULL;
+  }
+  
+  return librdf_new_storage_from_factory(world, factory, name, options_hash);
+}
+
+
 /**
  * librdf_new_storage_from_storage - Copy constructor - create a new librdf_storage object from an existing one
  * @old_storage: the existing storage &librdf_storage to use
@@ -431,7 +466,9 @@ int
 librdf_storage_add_statement(librdf_storage* storage,
                              librdf_statement* statement) 
 {
-  return storage->factory->add_statement(storage, statement);
+  if(storage->factory->add_statement)
+    return storage->factory->add_statement(storage, statement);
+  return 1;
 }
 
 
@@ -446,7 +483,9 @@ int
 librdf_storage_add_statements(librdf_storage* storage,
                               librdf_stream* statement_stream) 
 {
-  return storage->factory->add_statements(storage, statement_stream);
+  if(storage->factory->add_statements)
+    return storage->factory->add_statements(storage, statement_stream);
+  return 1;
 }
 
 
@@ -461,7 +500,9 @@ int
 librdf_storage_remove_statement(librdf_storage* storage, 
                                 librdf_statement* statement) 
 {
-  return storage->factory->remove_statement(storage, statement);
+  if(storage->factory->remove_statement)
+    return storage->factory->remove_statement(storage, statement);
+  return 1;
 }
 
 
@@ -916,10 +957,51 @@ librdf_storage_context_add_statement(librdf_storage* storage,
                                      librdf_node* context,
                                      librdf_statement* statement) 
 {
-  return storage->factory->context_add_statement(storage, context, statement);
+  if(storage->factory->context_add_statement)
+    return storage->factory->context_add_statement(storage, context, statement);
+  return 1;
 }
 
 
+/**
+ * librdf_storage_context_add_statements - Add statements to a storage with a context
+ * @storage: &librdf_storage object
+ * @context: &librdf_node context
+ * @stream: &librdf_stream stream object
+ * 
+ * Return value: Non 0 on failure
+ **/
+int
+librdf_storage_context_add_statements(librdf_storage* storage, 
+                                      librdf_node* context,
+                                      librdf_stream* stream) 
+{
+  int status=0;
+  
+  if(storage->factory->context_add_statements)
+    return storage->factory->context_add_statements(storage, context, stream);
+
+  if(!storage->factory->context_add_statement)
+    return 1;
+  
+  if(!stream)
+    return 1;
+
+  while(!librdf_stream_end(stream)) {
+    librdf_statement* statement=librdf_stream_get_object(stream);
+    if(!statement)
+      break;
+    status=librdf_storage_context_add_statement(storage, context, statement);
+    if(status)
+      break;
+    librdf_stream_next(stream);
+  }
+
+  return status;
+}
+
+
+
 /**
  * librdf_storage_context_remove_statement - Remove a statement from a storage in a context
  * @storage: &librdf_storage object
@@ -933,22 +1015,76 @@ librdf_storage_context_remove_statement(librdf_storage* storage,
                                         librdf_node* context,
                                         librdf_statement* statement) 
 {
+  if(!storage->factory->context_remove_statement)
+    return 1;
+  
   return storage->factory->context_remove_statement(storage, context, statement);
 }
 
 
 /**
- * librdf_storage_context_serialise - List all statements in a storage context
+ * librdf_storage_context_remove_statements - Remove statements from a storage with the given context
+ * @storage: &librdf_storage object
+ * @context: &librdf_uri context
+ * 
+ * Return value: Non 0 on failure
+ **/
+int
+librdf_storage_context_remove_statements(librdf_storage* storage,
+                                         librdf_node* context) 
+{
+  librdf_stream *stream;
+  if(storage->factory->context_remove_statements)
+    return storage->factory->context_remove_statements(storage, context);
+  
+  if(!storage->factory->context_remove_statement)
+    return 1;
+  
+  stream=librdf_storage_context_as_stream(storage, context);
+  if(!stream)
+    return 1;
+
+  while(!librdf_stream_end(stream)) {
+    librdf_statement *statement=librdf_stream_get_object(stream);
+    if(!statement)
+      break;
+    librdf_storage_context_remove_statement(storage, context, statement);
+    librdf_stream_next(stream);
+  }
+  librdf_free_stream(stream);  
+  return 0;
+}
+
+
+/**
+ * librdf_storage_context_as_stream - List all statements in a storage context
  * @storage: &librdf_storage object
  * @context: &librdf_node context node
  * 
  * Return value: &librdf_stream of statements or NULL on failure or context is empty
  **/
 librdf_stream*
+librdf_storage_context_as_stream(librdf_storage* storage, librdf_node* context)
+{
+  return storage->factory->context_serialise(storage, context);
+}
+
+
+/**
+ * librdf_storage_context_serialise - List all statements in a storage context (DEPRECATED)
+ * @storage: &librdf_storage object
+ * @context: &librdf_node context node
+ * 
+ * DEPRECATED to reduce confusion with the librdf_serializer class.
+ * Please use librdf_storage_context_as_stream.
+ *
+ * Return value: &librdf_stream of statements or NULL on failure or context is empty
+ **/
+librdf_stream*
 librdf_storage_context_serialise(librdf_storage* storage,
                                  librdf_node* context)
 {
-  return storage->factory->context_serialise(storage, context);
+  return librdf_storage_context_as_stream(storage, context);
 }
 
 
